@@ -1,9 +1,9 @@
 from .base import *
+from .db import *
 from .destroy import *
+from .init import *
 from .inspect import *
 from .launch import *
 from .list import *
-from .log import *
-from .server import *
 from .ssh import *
 from .status import *
